%% Clusters the locations into k clusters using the k-means method

%% Input

Sailingspeed = [13 13 10 10]; %Sailingspeed of the vessel in knots

Unloadingrate = [250 250 250 250]; %Unloading rate in tonnes/hour

%Number of clusters
nr_clusters = 30;

%Coordinates of the locations
Y = xlsread('Kmeans_Farms_Mid.xlsx','E3:F153');

%Factory coordinates
Factory = xlsread('Kmeans_Farms_Mid.xlsx','E2:F2');

% Reads in the demand of each location:
demand_loc = xlsread('Kmeans_Farms_Mid.xlsx','H3:H153');

%Coordinates limits of the plot
LATLIM = [57.5 71];
LONLIM = [0 23];



%% k-means algorithm

% Clusters the locations using the k-means method
% idx is the cluster number of each loaction
% C is the coordinates of each cluster center
% sumd is the sum of distances to the cluster center from each loaction
% D is the distance from each location to each cluster center
% NOTE: these distances are straight lines

% Partition the data into k clusters, and choose the best arrangement 
% out of ten intializations. Display the final output.

opts = statset('Display','final'); 
[idx,C, sumd] = kmeans(X,nr_clusters,'Replicates',10,'Options',opts);

%Creating a list of coordinates for the factory and the cluster centriods,
%can be readed into distance_cluster.m to find the distance matrix.


%Change position of lat and long
Coord = zeros(nr_clusters,2);

for i =1:nr_clusters
    Coord(i,1) = C(i,2);
    Coord(i,2) = C(i,1);
end

%Sorting the coordinates from north to south
B=sort(Coord,'descend');

D=zeros(nr_clusters,2);

for i=1:nr_clusters
    for j=1:nr_clusters
        if B(i,1) == Coord(j,1)
            D(i,1) = Coord(j,1);
            D(i,2) = Coord(j,2);
        end
    end
end

%Change the numberation of which cluster each farms belong to, in order to
%sort from north to south.

%Adds the latitude coordinate to cluster center belonging to the farm, so
%it can be sorted from north to south.
nr_farms = length(Y(:,1));

idx2=zeros(nr_farms,1);

for i=1:nr_farms
    for j=1:nr_clusters
        if idx(i) == j
            idx2(i) = Coord(j,1);
        end
    end
end

%Adds the new cluster number to each farm, so that the northest cluster is
%cluster 1, and all farms in that cluster has that number.
idx3=zeros(nr_farms,1);
for i=1:nr_farms
    for j=1:nr_clusters
        if idx2(i)==D(j,1)
            idx3(i) = j;
        end
    end
end


%Adding the factory to the coordinate list
Allcoords = zeros(nr_clusters+1,2);
Allcoords(1,:) = Factory; %Adds the factory to the first row in the cluster coordinates matrix

for i =1:nr_clusters
    Allcoords(i+1,1) = D(i,1);
    Allcoords(i+1,2) = D(i,2);
end


save('Allcoords.mat','Allcoords');
%% Plots the locations and the cluster centroids 


figure;
hold on
ax = worldmap(LATLIM,LONLIM);  % Create map figure

legendInfo{1} = ['Coastline'];
geoshow('landareas.shp','FaceColor', [1 1 1]);
colors=parula(nr_clusters);
for i=1:nr_clusters %Plots the farm locations
    plotm(X(idx3==i,2),X(idx3==i,1),'.','MarkerSize',10,'color',colors(i,:));
    legendInfo{i+1} = ['Cluster' num2str(i)];
end
plotm(Factory(1,1),Factory(1,2),'*','MarkerSize',8,'color','k','LineWidth',2); %Plots Factory location
legendInfo{nr_clusters+2} = ['Factory'];
plotm(D(:,1),D(:,2),'kx','MarkerSize',10,'LineWidth',3); %Plots Cluster centroids
legendInfo{nr_clusters+3} = ['Cluster Center'];
legend(legendInfo,'Location','NW')
hold off


%% Calculating the demand of each cluster and number of farms in each cluster


demand_clust = zeros(1,nr_clusters); %Matrix of demand in each cluster
farm_in_clust = zeros(1,nr_clusters); %Matrix of number of farms in each cluster


% Finds the total demand of each cluster
for i=1:length(demand_loc)
    for j=1:nr_clusters
        if idx3(i) == j
            demand_clust(j)=demand_clust(j)+demand_loc(i);
            farm_in_clust(j) = farm_in_clust(j) + 1;
        else
            demand_clust(j)=demand_clust(j);
            farm_in_clust(j) = farm_in_clust(j);
        end
    end
end

capacity_clust = farm_in_clust * 300; %Generates the capacity of each cluster
charter_price = farm_in_clust * 1000; %Generates the charter price of each cluster
%% Creates total distance in each cluster from cluster center to all nodes in each cluster


arclen = zeros(nr_farms,nr_clusters);
Totdist = zeros(nr_clusters,1);

%Finds the distance from each node to each cluster center
 for i = 1:nr_clusters
     for j = 1:nr_farms
         arclen(j,i) = deg2nm(distance('gc',D(i,1), D(i,2), Y(j,1), Y(j,2)));
     end
 end
 
 %Sums the total distance within each cluster
 for i = 1:nr_farms
     for j = 1:nr_clusters
         if idx3(i) == j
             Totdist(j) = Totdist(j) + arclen(i,j);
         else
             Totdist(j) = Totdist(j);
         end
     end
 end


%% Calculate time in cluster:

%Total timeusage in each cluster
Tot_time_cluster = zeros(nr_clusters,length(Sailingspeed));

for i=1:nr_clusters
    for j=1:length(Sailingspeed)
Tot_time_cluster(i,j) = ((Totdist(i)/Sailingspeed(j))*0.5) + (demand_clust(i)/Unloadingrate(j));
    end
end