%% Input

nr_clusters = 30; %Number of clusters

nr_farms = 151; %Number of fish farms

Sailingspeed = [8 8 8 8]; %Sailingspeed in knots

Unloadingrate = [250 250 250 250]; %Unloading rate in tonnes/hour

%Coordinates of cluster centroids:
%C = xlsread('Contamination_Zones_Centroids_Coordinates.xlsx','B3:C20');

%Coodinates of the factory:
F = xlsread('Kmeans_Farms_Low.xlsx','E2:F2');

%Coordnates of farms:
Y = xlsread('Kmeans_Farms_Low.xlsx','E3:F153');

%Cluster number of each farm.
cluster_nr = xlsread('Kmeans_Farms_Low.xlsx','G3:G153');


% Reads in the demand of each location:
demand_loc = xlsread('Kmeans_Farms_Low.xlsx','H3:H153');



%% Calculating the demand of each cluster and number of farms in each cluster


demand_clust = zeros(1,nr_clusters); %Matrix of demand in each cluster
farm_in_clust = zeros(1,nr_clusters); %Matrix of number of farms in each cluster

% Finds the total demand of each cluster
for i=1:length(demand_loc)
    for j=1:nr_clusters
        if cluster_nr(i) == j
            demand_clust(j)=demand_clust(j)+demand_loc(i);
            farm_in_clust(j) = farm_in_clust(j) + 1;
        else
            demand_clust(j)=demand_clust(j);
            farm_in_clust(j) = farm_in_clust(j);
        end
    end
end

capacity_clust = farm_in_clust * 300; %Generates the capacity of each cluster
charter_price = farm_in_clust * 1000; %Generates the charter price of each cluster
%% Find average coordinates in each cluster

%Zero matrices
lat = zeros(nr_clusters,1);
lon = zeros(nr_clusters,1);
av_lat = zeros(nr_clusters,1);
av_lon = zeros(nr_clusters,1);

%Adds the coordinate of each farm in each cluster together
for i=1:nr_farms
    for j=1:nr_clusters
        if cluster_nr(i) == j
            lat(j) = lat(j) + Y(i,1);
            lon(j) = lon(j) + Y(i,2);
        else
            lat(j) = lat(j);
            lon(j) = lon(j);
        end
    end
end

%Finds the average of the coordinates of each cluster
for i=1:nr_clusters
    if lat(i) == 0
        av_lat(i) = 0;
        av_lon(i) = 0;
    else
        av_lat(i) = lat(i) / farm_in_clust(i);
        av_lon(i) = lon(i) / farm_in_clust(i);
    end
end

%Coordinates of cluster centroids
C= [av_lat,av_lon];

%Adding factory to coordinate list
Coord = zeros(nr_clusters+1,2);
Coord(1,:) = F; %Adds the factory to the first row in the cluster coordinates matrix

for i =1:nr_clusters
    Coord(i+1,:) = [av_lat(i),av_lon(i)];
end


%Change cluster position if some cluster positions are not working
%Coord(14,:) = [63.790,8.480];
%Coord(16,:) = [63.717,8.511];

%% Creates total distance in each cluster from cluster center to all nodes in each cluster


arclen = zeros(nr_farms,nr_clusters);
Totdist = zeros(nr_clusters,1);

%Finds the distance from each node to each cluster center
 for i = 1:nr_clusters
     for j = 1:nr_farms
         arclen(j,i) = deg2nm(distance('gc',C(i,1), C(i,2), Y(j,1), Y(j,2)));
     end
 end
 
 %Sums the total distance within each cluster
 for i = 1:nr_farms
     for j = 1:nr_clusters
         if cluster_nr(i) == j
             Totdist(j) = Totdist(j) + arclen(i,j);
         else
             Totdist(j) = Totdist(j);
         end
     end
 end
%% Calculate time in cluster:

%Total timeusage in each cluster (in hours)
Tot_time_cluster = zeros(nr_clusters,length(Sailingspeed));

for i=1:nr_clusters
    for j=1:length(Sailingspeed)
Tot_time_cluster(i,j) = (Totdist(i)/Sailingspeed(j)*0.5) + (demand_clust(i)/Unloadingrate(j));
    end
end

%% Save coordinates for use in the distance_cluster script

Allcoords = Coord;
save('Allcoords.mat','Allcoords');
